@@ -1,4 +1,7 @@
-/*  BMP - Cross-platform multimedia player
+/*  Audacious - Cross-platform multimedia player
+ *  Copyright (C) 2005-2006  Audacious development team.
+ *
+ *  BMP - Cross-platform multimedia player
  *  Copyright (C) 2003-2004  BMP development team.
  *
  *  Based on XMMS:
@@ -16,7 +19,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include "ui_playlist.h"
@@ -28,9 +31,8 @@
 #include <gtk/gtk.h>
 #include <string.h>
 
-#include <gdk/gdkx.h>
+#include "platform/smartinclude.h"
 
-#include <X11/Xlib.h>
 #include <unistd.h>
 #include <errno.h>
 
@@ -45,13 +47,7 @@
 #include "mainwin.h"
 #include "playback.h"
 #include "playlist.h"
-#include "playlist_list.h"
-#include "playlist_slider.h"
-#include "playlist_popup.h"
-#include "pbutton.h"
-#include "sbutton.h"
-#include "skin.h"
-#include "textbox.h"
+#include "playlist_container.h"
 #include "util.h"
 
 #include "pixmaps.h"
@@ -71,15 +67,18 @@ enum {
     SEL_LOOKUP, CLOSE_PL_WINDOW, MOVE_UP, PLIST_SAVE,
     MISC_QUEUE, PLIST_CQUEUE, PLIST_JTF, PLIST_JTT,
     PLAYLISTWIN_REMOVE_DEAD_FILES,
-    PLAYLISTWIN_REFRESH
+    PLAYLISTWIN_REFRESH, PLIST_DEFAULTSAVE, MISC_FILEPOPUP
 };
 
 enum {
     PLAYLISTWIN_SORT_BYTITLE, PLAYLISTWIN_SORT_BYFILENAME,
     PLAYLISTWIN_SORT_BYPATH, PLAYLISTWIN_SORT_BYDATE,
+    PLAYLISTWIN_SORT_BYARTIST, PLAYLISTWIN_SORT_SEL_BYARTIST,
     PLAYLISTWIN_SORT_SEL_BYTITLE, PLAYLISTWIN_SORT_SEL_BYFILENAME,
     PLAYLISTWIN_SORT_SEL_BYPATH, PLAYLISTWIN_SORT_SEL_BYDATE,
-    PLAYLISTWIN_SORT_RANDOMIZE, PLAYLISTWIN_SORT_REVERSE
+    PLAYLISTWIN_SORT_RANDOMIZE, PLAYLISTWIN_SORT_REVERSE,
+    PLAYLISTWIN_SORT_BYTRACK, PLAYLISTWIN_SORT_SEL_BYTRACK,
+    PLAYLISTWIN_SORT_BYPLAYLIST, PLAYLISTWIN_SORT_SEL_BYPLAYLIST
 };
 
 GtkWidget *playlistwin;
@@ -124,6 +123,10 @@ static GtkItemFactoryEntry playlistwin_popup_menu_entries[] = {
      playlistwin_popup_menu_callback,
      MISC_FILEINFO, "<ImageItem>", my_pixbuf},
 
+    {N_("/Show Popup Info"), NULL,
+     playlistwin_popup_menu_callback,
+     MISC_FILEPOPUP, "<ToggleItem>", NULL},
+
     ITEM_SEPARATOR,
 
     {N_("/Remove Selected"), "Delete",
@@ -197,7 +200,7 @@ static GtkItemFactoryEntry pldel_menu_entries[] = {
 };
 
 static GtkItemFactoryEntry pllist_menu_entries[] = {
-    {N_("/New List"), NULL,
+    {N_("/New List"), "<shift>N",
      playlistwin_sub_menu_callback,
      PLIST_NEW, "<StockItem>", GTK_STOCK_NEW},
 
@@ -207,10 +210,14 @@ static GtkItemFactoryEntry pllist_menu_entries[] = {
      playlistwin_sub_menu_callback,
      PLIST_LOAD, "<StockItem>", GTK_STOCK_OPEN},
 
-    {N_("/Save List"), "s",
+    {N_("/Save List"), "<shift>S",
      playlistwin_sub_menu_callback,
      PLIST_SAVE, "<StockItem>", GTK_STOCK_SAVE},
 
+    {N_("/Save Default List"), "<alt>S",
+     playlistwin_sub_menu_callback,
+     PLIST_DEFAULTSAVE, "<StockItem>", GTK_STOCK_SAVE},
+
     ITEM_SEPARATOR,
 
     {N_("/Update View"), "F5",
@@ -235,7 +242,7 @@ static GtkItemFactoryEntry plsel_menu_entries[] = {
 };
 
 static GtkItemFactoryEntry plsort_menu_entries[] = {
-    {N_("/Randomize List"), NULL, plsort_menu_callback,
+    {N_("/Randomize List"), "<Ctrl><Shift>R", plsort_menu_callback,
      PLAYLISTWIN_SORT_RANDOMIZE, "<ImageItem>", randomizepl_pixbuf},
     {N_("/Reverse List"), NULL, plsort_menu_callback,
      PLAYLISTWIN_SORT_REVERSE, "<ImageItem>", invertpl_pixbuf},
@@ -243,21 +250,33 @@ static GtkItemFactoryEntry plsort_menu_entries[] = {
     {N_("/Sort List"), NULL, NULL, 0, "<Branch>", NULL},
     {N_("/Sort List/By Title"), NULL, plsort_menu_callback,
      PLAYLISTWIN_SORT_BYTITLE, "<ImageItem>", sortbytitle_pixbuf},
+    {N_("/Sort List/By Artist"), NULL, plsort_menu_callback,
+     PLAYLISTWIN_SORT_BYARTIST, "<ImageItem>", sortbytitle_pixbuf},
     {N_("/Sort List/By Filename"), NULL, plsort_menu_callback,
      PLAYLISTWIN_SORT_BYFILENAME, "<ImageItem>", sortbyfilename_pixbuf},
     {N_("/Sort List/By Path + Filename"), NULL, plsort_menu_callback,
      PLAYLISTWIN_SORT_BYPATH, "<ImageItem>", sortbypathfile_pixbuf},
     {N_("/Sort List/By Date"), NULL, plsort_menu_callback,
      PLAYLISTWIN_SORT_BYDATE, "<ImageItem>", sortbydate_pixbuf},
+    {N_("/Sort List/By Track Number"), NULL, plsort_menu_callback,
+     PLAYLISTWIN_SORT_BYTRACK, "<ImageItem>", sortbytitle_pixbuf},
+    {N_("/Sort List/By Playlist Entry"), NULL, plsort_menu_callback,
+     PLAYLISTWIN_SORT_BYPLAYLIST, "<ImageItem>", sortbytitle_pixbuf},
     {N_("/Sort Selection"), NULL, NULL, 0, "<Branch>", NULL},
     {N_("/Sort Selection/By Title"), NULL, plsort_menu_callback,
      PLAYLISTWIN_SORT_SEL_BYTITLE, "<ImageItem>", sortbytitle_pixbuf},
+    {N_("/Sort Selection/By Artist"), NULL, plsort_menu_callback,
+     PLAYLISTWIN_SORT_SEL_BYARTIST, "<ImageItem>", sortbytitle_pixbuf},
     {N_("/Sort Selection/By Filename"), NULL, plsort_menu_callback,
      PLAYLISTWIN_SORT_SEL_BYFILENAME, "<ImageItem>", sortbyfilename_pixbuf},
     {N_("/Sort Selection/By Path + Filename"), NULL, plsort_menu_callback,
      PLAYLISTWIN_SORT_SEL_BYPATH, "<ImageItem>", sortbypathfile_pixbuf},
     {N_("/Sort Selection/By Date"), NULL, plsort_menu_callback,
-     PLAYLISTWIN_SORT_SEL_BYDATE, "<ImageItem>", sortbydate_pixbuf}
+     PLAYLISTWIN_SORT_SEL_BYDATE, "<ImageItem>", sortbydate_pixbuf},
+    {N_("/Sort Selection/By Track Number"), NULL, plsort_menu_callback,
+     PLAYLISTWIN_SORT_SEL_BYTRACK, "<ImageItem>", sortbytitle_pixbuf},
+    {N_("/Sort Selection/By Playlist Entry"), NULL, plsort_menu_callback,
+     PLAYLISTWIN_SORT_SEL_BYPLAYLIST, "<ImageItem>", sortbytitle_pixbuf}
 };
 
 
@@ -273,12 +292,16 @@ playlistwin_is_shaded(void)
 gint
 playlistwin_get_width(void)
 {
+    cfg.playlist_width /= PLAYLISTWIN_WIDTH_SNAP;
+    cfg.playlist_width *= PLAYLISTWIN_WIDTH_SNAP;
     return cfg.playlist_width;
 }
 
 gint
 playlistwin_get_height_unshaded(void)
 {
+    cfg.playlist_height /= PLAYLISTWIN_HEIGHT_SNAP;
+    cfg.playlist_height *= PLAYLISTWIN_HEIGHT_SNAP;
     gint height = cfg.playlist_height;
     return height;
 }
@@ -344,7 +367,7 @@ playlistwin_update_info(void)
     else
         tot_text = g_strdup("?");
     text = g_strconcat(sel_text, "/", tot_text, NULL);
-    textbox_set_text(playlistwin_info, text);
+    textbox_set_text(playlistwin_info, text ? text : "");
     g_free(text);
     g_free(tot_text);
     g_free(sel_text);
@@ -353,50 +376,40 @@ playlistwin_update_info(void)
 static void
 playlistwin_update_sinfo(void)
 {
-    gchar *posstr, *timestr, *title, *info, *dots;
+    gchar *posstr, *timestr, *title, *info;
     gint pos, time;
-    guint max_len;
 
     pos = playlist_get_position();
     title = playlist_get_songtitle(pos);
-    time = playlist_get_songtime(pos);
 
     if (!title) {
         textbox_set_text(playlistwin_sinfo, "");
         return;
     }
 
+    convert_title_text(title);
+
+    time = playlist_get_songtime(pos);
+
     if (cfg.show_numbers_in_pl)
         posstr = g_strdup_printf("%d. ", pos + 1);
     else
         posstr = g_strdup("");
 
-    max_len = (playlistwin_get_width() - 35) / 5 - strlen(posstr);
-
     if (time != -1) {
-        timestr = g_strdup_printf(" %d:%-2.2d", time / 60000,
-                                  (time / 1000) % 60);
-        max_len -= strlen(timestr);
+        timestr = g_strdup_printf(" (%d:%-2.2d)", time / 60000,
+                                      (time / 1000) % 60);
     }
     else
         timestr = g_strdup("");
 
-    convert_title_text(title);
+    info = g_strdup_printf("%s%s%s", posstr, title, timestr);
 
-    if (strlen(title) > max_len) {
-        max_len -= 3;
-        dots = "...";
-    }
-    else
-        dots = "";
-
-    info = g_strdup_printf("%s%-*.*s%s%s", posstr, max_len, max_len,
-                           title, dots, timestr);
     g_free(posstr);
     g_free(title);
     g_free(timestr);
 
-    textbox_set_text(playlistwin_sinfo, info);
+    textbox_set_text(playlistwin_sinfo, info ? info : "");
     g_free(info);
 }
 
@@ -479,7 +492,6 @@ playlistwin_set_geometry_hints(gboolean shaded)
 
     geometry.min_width = PLAYLISTWIN_MIN_WIDTH;
     geometry.max_width = G_MAXUINT16;
-    geometry.base_width = cfg.playlist_width;
 
     geometry.width_inc = PLAYLISTWIN_WIDTH_SNAP;
     geometry.height_inc = PLAYLISTWIN_HEIGHT_SNAP;
@@ -492,22 +504,54 @@ playlistwin_set_geometry_hints(gboolean shaded)
     else {
         geometry.min_height = PLAYLISTWIN_MIN_HEIGHT;
         geometry.max_height = G_MAXUINT16;
-        geometry.base_height = cfg.playlist_height;
     }
 
-    mask = GDK_HINT_MIN_SIZE | GDK_HINT_MAX_SIZE | GDK_HINT_RESIZE_INC |
-           GDK_HINT_BASE_SIZE;
+    mask = GDK_HINT_MIN_SIZE | GDK_HINT_MAX_SIZE | GDK_HINT_RESIZE_INC;
 
     gtk_window_set_geometry_hints(GTK_WINDOW(playlistwin),
                                   playlistwin, &geometry, mask);
 }
 
+void
+playlistwin_set_sinfo_font(gchar *font)
+{
+    gchar *tmp = NULL, *tmp2 = NULL;
+
+    if(!font)
+        return;
+
+    tmp = g_strdup(font);
+    if(!tmp)
+        return;
+
+    *strrchr(tmp, ' ') = '\0';
+    tmp2 = g_strdup_printf("%s 8", tmp);
+    if(!tmp2)
+        return;
+    textbox_set_xfont(playlistwin_sinfo, cfg.mainwin_use_xfont, tmp2);
+
+    g_free(tmp); g_free(tmp2);
+}
+
+void
+playlistwin_set_sinfo_scroll(gboolean scroll)
+{
+    GtkWidget *item;
+
+    if(playlistwin_is_shaded())
+        textbox_set_scroll(playlistwin_sinfo, cfg.autoscroll);
+    else
+        textbox_set_scroll(playlistwin_sinfo, FALSE);
+}
+
 void
 playlistwin_set_shade(gboolean shaded)
 {
     cfg.playlist_shaded = shaded;
 
     if (shaded) {
+        playlistwin_set_sinfo_font(cfg.playlist_font);
+        playlistwin_set_sinfo_scroll(cfg.autoscroll);
         widget_show(WIDGET(playlistwin_sinfo));
         playlistwin_shade->pb_nx = 128;
         playlistwin_shade->pb_ny = 45;
@@ -518,6 +562,7 @@ playlistwin_set_shade(gboolean shaded)
     }
     else {
         widget_hide(WIDGET(playlistwin_sinfo));
+        playlistwin_set_sinfo_scroll(FALSE);
         playlistwin_shade->pb_nx = 157;
         playlistwin_shade->pb_ny = 3;
         playlistwin_shade->pb_px = 62;
@@ -532,7 +577,7 @@ playlistwin_set_shade(gboolean shaded)
     playlistwin_set_geometry_hints(cfg.playlist_shaded);
 
     gtk_window_resize(GTK_WINDOW(playlistwin),
-                      cfg.playlist_width,
+                      playlistwin_get_width(),
                       playlistwin_get_height());
 
     playlistwin_set_mask();
@@ -553,6 +598,7 @@ playlistwin_set_shade_menu(gboolean shaded)
     gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(item), shaded);
 
     playlistwin_set_shade(shaded);
+    playlistwin_update_list();
 }
 
 void
@@ -572,18 +618,18 @@ playlistwin_release(GtkWidget * widget,
     gdk_pointer_ungrab(GDK_CURRENT_TIME);
     playlistwin_resizing = FALSE;
     gdk_flush();
+ 
+    if (dock_is_moving(GTK_WINDOW(playlistwin)))
+    {
+       dock_move_release(GTK_WINDOW(playlistwin));
 
-    if (dock_is_moving(GTK_WINDOW(playlistwin))) {
-        dock_move_release(GTK_WINDOW(playlistwin));
-#if 0
-        if (cfg.playlist_transparent)
-            playlistwin_update_list();
-#endif
+       if (cfg.playlist_transparent)
+           playlistwin_update_list();
     }
-    else {
-        handle_release_cb(playlistwin_wlist, widget, event);
-        playlist_popup_destroy();
-        draw_playlist_window(FALSE);
+    else
+    {
+       handle_release_cb(playlistwin_wlist, widget, event);
+       draw_playlist_window(FALSE);
     }
 }
 
@@ -713,6 +759,22 @@ playlistwin_motion(GtkWidget * widget,
     while ((gevent = gdk_event_get()) != NULL) gdk_event_free(gevent);
 }
 
+static void
+playlistwin_enter(GtkWidget * widget,
+                   GdkEventMotion * event,
+                   gpointer callback_data)
+{
+    playlistwin_list->pl_tooltips = TRUE;
+}
+
+static void
+playlistwin_leave(GtkWidget * widget,
+                   GdkEventMotion * event,
+                   gpointer callback_data)
+{
+    playlistwin_list->pl_tooltips = FALSE;
+}
+
 static void
 playlistwin_show_filebrowser(void)
 {
@@ -833,10 +895,11 @@ show_playlist_save_format_error(GtkWindow * parent,
 static void
 playlistwin_save_playlist(const gchar * filename)
 {
-    PlaylistFormat format;
+    PlaylistContainer *plc;
+    gchar *ext = strrchr(filename, '.') + 1;
 
-    format = playlist_format_get_from_name(filename);
-    if (format == PLAYLIST_FORMAT_UNKNOWN) {
+    plc = playlist_container_find(ext);
+    if (plc == NULL) {
         show_playlist_save_format_error(GTK_WINDOW(playlistwin), filename);
         return;
     }
@@ -847,7 +910,7 @@ playlistwin_save_playlist(const gchar * filename)
         if (!show_playlist_overwrite_prompt(GTK_WINDOW(playlistwin), filename))
             return;
 
-    if (!playlist_save(filename, format))
+    if (!playlist_save(filename))
         show_playlist_save_error(GTK_WINDOW(playlistwin), filename);
 }
 
@@ -880,40 +943,80 @@ playlistwin_load_playlist(const gchar * filename)
 }
 
 static gchar *
-playlist_file_selection(const gchar * title,
-                        gboolean save,
+playlist_file_selection_load(const gchar * title,
                         const gchar * default_filename)
 {
-    GtkWidget *dialog, *button;
+    static GtkWidget *dialog = NULL;
+    GtkWidget *button;
     gchar *filename;
 
     g_return_val_if_fail(title != NULL, NULL);
 
-    dialog = gtk_file_chooser_dialog_new(title, GTK_WINDOW(mainwin),
-                                         save ? GTK_FILE_CHOOSER_ACTION_SAVE :
-                                         GTK_FILE_CHOOSER_ACTION_OPEN, NULL, NULL);
-    if (default_filename)
-        gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(dialog),
-                                      default_filename);
+    if(!dialog) {
+        dialog = gtk_file_chooser_dialog_new(title, GTK_WINDOW(mainwin),
+                                             GTK_FILE_CHOOSER_ACTION_OPEN, NULL, NULL);
 
-    button = gtk_dialog_add_button(GTK_DIALOG(dialog), GTK_STOCK_CANCEL,
-                                   GTK_RESPONSE_REJECT);
-    gtk_button_set_use_stock(GTK_BUTTON(button), TRUE);
-    GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
+        if (default_filename)
+            gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(dialog),
+                                          default_filename);
 
-    button = gtk_dialog_add_button(GTK_DIALOG(dialog),
-                                   save ? GTK_STOCK_SAVE : GTK_STOCK_OPEN,
-                                   GTK_RESPONSE_ACCEPT);
-    gtk_button_set_use_stock(GTK_BUTTON(button), TRUE);
-    gtk_dialog_set_default_response(GTK_DIALOG(dialog), GTK_RESPONSE_ACCEPT);
+        button = gtk_dialog_add_button(GTK_DIALOG(dialog), GTK_STOCK_CANCEL,
+                                       GTK_RESPONSE_REJECT);
+        gtk_button_set_use_stock(GTK_BUTTON(button), TRUE);
+        GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
+
+        button = gtk_dialog_add_button(GTK_DIALOG(dialog),
+                                       GTK_STOCK_OPEN,
+                                       GTK_RESPONSE_ACCEPT);
+        gtk_button_set_use_stock(GTK_BUTTON(button), TRUE);
+        gtk_dialog_set_default_response(GTK_DIALOG(dialog), GTK_RESPONSE_ACCEPT);
+    }
 
     if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT)
         filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
     else
         filename = NULL;
 
-    gtk_widget_destroy(dialog);
+    gtk_widget_hide(dialog);
+    return filename;
+}
 
+static gchar *
+playlist_file_selection_save(const gchar * title,
+                        const gchar * default_filename)
+{
+    static GtkWidget *dialog = NULL;
+    GtkWidget *button;
+    gchar *filename;
+
+    g_return_val_if_fail(title != NULL, NULL);
+
+    if(!dialog) {
+        dialog = gtk_file_chooser_dialog_new(title, GTK_WINDOW(mainwin),
+                                             GTK_FILE_CHOOSER_ACTION_SAVE, NULL, NULL);
+
+        if (default_filename)
+            gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(dialog),
+                                          default_filename);
+
+        button = gtk_dialog_add_button(GTK_DIALOG(dialog), GTK_STOCK_CANCEL,
+                                       GTK_RESPONSE_REJECT);
+        gtk_button_set_use_stock(GTK_BUTTON(button), TRUE);
+        GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
+
+        button = gtk_dialog_add_button(GTK_DIALOG(dialog),
+                                       GTK_STOCK_SAVE,
+                                       GTK_RESPONSE_ACCEPT);
+        gtk_button_set_use_stock(GTK_BUTTON(button), TRUE);
+        gtk_dialog_set_default_response(GTK_DIALOG(dialog), GTK_RESPONSE_ACCEPT);
+    }
+
+    if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT)
+        filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
+    else
+        filename = NULL;
+
+    gtk_widget_hide(dialog);
     return filename;
 }
 
@@ -921,7 +1024,7 @@ void
 playlistwin_select_playlist_to_load(const gchar * default_filename)
 {
     gchar *filename =
-        playlist_file_selection(_("Load Playlist"), FALSE, default_filename);
+        playlist_file_selection_load(_("Load Playlist"), default_filename);
 
     if (filename) {
         playlistwin_load_playlist(filename);
@@ -932,19 +1035,20 @@ playlistwin_select_playlist_to_load(const gchar * default_filename)
 static void
 playlistwin_select_playlist_to_save(const gchar * default_filename)
 {
+    gchar *dot = NULL, *basename = NULL;
     gchar *filename =
-        playlist_file_selection(_("Save Playlist"), TRUE, default_filename);
+        playlist_file_selection_save(_("Save Playlist"), default_filename);
 
     if (filename) {
-        /* Default to M3U if no filename has extension */
-
-        /* NOTE: This doesn't work correctly for hidden files
-           - descender */
-        if (!strchr(filename, '.')) {
-            gchar *tmpstr = filename;
-            filename = g_strconcat(filename, ".m3u", NULL);
-            g_free(tmpstr);
+        /* Default to xspf if no filename has extension */
+        basename = g_path_get_basename(filename);
+        dot = strrchr(basename, '.');
+        if( dot == NULL || dot == basename) {
+            gchar *oldname = filename;
+            filename = g_strconcat(oldname, ".xspf", NULL);
+            g_free(oldname);
         }
+        g_free(basename);
 
         playlistwin_save_playlist(filename);
         g_free(filename);
@@ -1101,10 +1205,11 @@ playlistwin_press(GtkWidget * widget,
                               GDK_TARGET_STRING, event->time);
     }
     else if (event->button == 1 && event->type == GDK_BUTTON_PRESS &&
-             !inside_sensitive_widgets(event->x, event->y) && event->y < 14) {
-        gdk_window_raise(playlistwin->window);
+             !inside_sensitive_widgets(event->x, event->y) && (cfg.easy_move || event->y < 14))
+    {
         dock_move_press(dock_window_list, GTK_WINDOW(playlistwin), event,
                         FALSE);
+        gtk_window_present(GTK_WINDOW(playlistwin));
     }
     else if (event->button == 1 && event->type == GDK_2BUTTON_PRESS &&
              !inside_sensitive_widgets(event->x, event->y)
@@ -1137,6 +1242,10 @@ playlistwin_press(GtkWidget * widget,
              widget_contains(WIDGET(playlistwin_list), event->x, event->y)) {
         /* popup menu */
         playlistwin_set_sensitive_sortmenu();
+        {
+            GtkWidget *item = gtk_item_factory_get_widget(playlistwin_popup_menu, "/Show Popup Info");
+            gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(item), cfg.show_filepopup_for_tuple);
+        }
         gtk_item_factory_popup(playlistwin_popup_menu,
                                event->x_root, event->y_root + 5,
                                3, event->time);
@@ -1503,12 +1612,15 @@ playlistwin_drag_data_received(GtkWidget * widget,
 static void
 playlistwin_create_widgets(void)
 {
+    gchar *font = NULL, *tmp = NULL;
     /* This function creates the custom widgets used by the playlist editor */
 
     /* text box for displaying song title in shaded mode */
     playlistwin_sinfo =
         create_textbox(&playlistwin_wlist, playlistwin_bg, playlistwin_gc,
-                       4, 4, playlistwin_get_width() - 35, FALSE, SKIN_TEXT);
+                       4, 4, playlistwin_get_width() - 35, TRUE, SKIN_TEXT);
+
+    playlistwin_set_sinfo_font(cfg.playlist_font);
 
     if (!cfg.playlist_shaded)
         widget_hide(WIDGET(playlistwin_sinfo));
@@ -1567,7 +1679,7 @@ playlistwin_create_widgets(void)
     playlistwin_info =
         create_textbox(&playlistwin_wlist, playlistwin_bg, playlistwin_gc,
                        playlistwin_get_width() - 143,
-                       cfg.playlist_height - 28, 85, FALSE, SKIN_TEXT);
+                       cfg.playlist_height - 28, 90, FALSE, SKIN_TEXT);
 
     /* mini play control buttons at right bottom corner */
 
@@ -1637,6 +1749,7 @@ playlistwin_create_window(void)
 
     playlistwin = gtk_window_new(GTK_WINDOW_TOPLEVEL);
     gtk_window_set_title(GTK_WINDOW(playlistwin), _("Audacious Playlist Editor"));
+    gtk_window_set_wmclass(GTK_WINDOW(playlistwin), "playlist", "Audacious");
     gtk_window_set_role(GTK_WINDOW(playlistwin), "playlist");
     gtk_window_set_default_size(GTK_WINDOW(playlistwin),
                                 playlistwin_get_width(),
@@ -1661,7 +1774,7 @@ playlistwin_create_window(void)
         gtk_window_move(GTK_WINDOW(playlistwin),
                         cfg.playlist_x, cfg.playlist_y);
 
-    gtk_widget_add_events(playlistwin,
+    gtk_widget_add_events(playlistwin, GDK_POINTER_MOTION_MASK |
                           GDK_FOCUS_CHANGE_MASK | GDK_BUTTON_MOTION_MASK |
                           GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK |
                           GDK_SCROLL_MASK | GDK_VISIBILITY_NOTIFY_MASK);
@@ -1679,6 +1792,10 @@ playlistwin_create_window(void)
                      G_CALLBACK(playlistwin_scrolled), NULL);
     g_signal_connect(playlistwin, "motion_notify_event",
                      G_CALLBACK(playlistwin_motion), NULL);
+    g_signal_connect(playlistwin, "enter_notify_event",
+                     G_CALLBACK(playlistwin_enter), NULL);
+    g_signal_connect(playlistwin, "leave_notify_event",
+                     G_CALLBACK(playlistwin_leave), NULL);
     g_signal_connect_after(playlistwin, "focus_in_event",
                            G_CALLBACK(playlistwin_focus_in), NULL);
     g_signal_connect_after(playlistwin, "focus_out_event",
@@ -1807,10 +1924,22 @@ plsort_menu_callback(gpointer data,
                      GtkWidget * widget)
 {
     switch (action) {
+    case PLAYLISTWIN_SORT_BYPLAYLIST:
+        playlist_sort(PLAYLIST_SORT_PLAYLIST);
+        playlistwin_update_list();
+        break;
+    case PLAYLISTWIN_SORT_BYTRACK:
+        playlist_sort(PLAYLIST_SORT_TRACK);
+        playlistwin_update_list();
+        break;
     case PLAYLISTWIN_SORT_BYTITLE:
         playlist_sort(PLAYLIST_SORT_TITLE);
         playlistwin_update_list();
         break;
+    case PLAYLISTWIN_SORT_BYARTIST:
+        playlist_sort(PLAYLIST_SORT_ARTIST);
+        playlistwin_update_list();
+        break;
     case PLAYLISTWIN_SORT_BYPATH:
         playlist_sort(PLAYLIST_SORT_PATH);
         playlistwin_update_list();
@@ -1823,10 +1952,22 @@ plsort_menu_callback(gpointer data,
         playlist_sort(PLAYLIST_SORT_FILENAME);
         playlistwin_update_list();
         break;
+    case PLAYLISTWIN_SORT_SEL_BYPLAYLIST:
+        playlist_sort_selected(PLAYLIST_SORT_PLAYLIST);
+        playlistwin_update_list();
+        break;
+    case PLAYLISTWIN_SORT_SEL_BYTRACK:
+        playlist_sort_selected(PLAYLIST_SORT_TRACK);
+        playlistwin_update_list();
+        break;
     case PLAYLISTWIN_SORT_SEL_BYTITLE:
         playlist_sort_selected(PLAYLIST_SORT_TITLE);
         playlistwin_update_list();
         break;
+    case PLAYLISTWIN_SORT_SEL_BYARTIST:
+        playlist_sort_selected(PLAYLIST_SORT_ARTIST);
+        playlistwin_update_list();
+        break;
     case PLAYLISTWIN_SORT_SEL_BYFILENAME:
         playlist_sort_selected(PLAYLIST_SORT_FILENAME);
         playlistwin_update_list();
@@ -1865,6 +2006,9 @@ playlistwin_sub_menu_callback(gpointer data,
     case PLIST_SAVE:
         playlistwin_select_playlist_to_save(playlist_get_current_name());
         break;
+    case PLIST_DEFAULTSAVE:
+        playlist_save(bmp_paths[BMP_PATH_PLAYLIST_FILE]);
+        break;
     case PLIST_SAVE_AS:
         playlistwin_select_playlist_to_save(playlist_get_current_name());
         break;
@@ -1915,6 +2059,8 @@ playlistwin_popup_menu_callback(gpointer data,
                                 guint action,
                                 GtkWidget * widget)
 {
+    extern GtkWidget *filepopupbutton;
+
     switch (action) {
     case ADD_FILES:
         playlistwin_show_filebrowser();
@@ -1940,5 +2086,12 @@ playlistwin_popup_menu_callback(gpointer data,
     case PLIST_JTF:
         mainwin_jump_to_file();
         break;
+    case MISC_FILEPOPUP:
+        cfg.show_filepopup_for_tuple = gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(widget));
+        if(filepopupbutton != NULL){
+            gtk_signal_emit_by_name(GTK_OBJECT(filepopupbutton), "realize");
+        }
+        break;
+
     }
 }
